@@ -25,6 +25,10 @@ class Game(sge.Game):
     def event_key_press(self, key, char):
         if key == 'escape':
             self.end()
+        elif key == "backspace":
+            view = self.current_room.views[0]
+            view.width = view.wport
+            view.height = view.hport
 
     def event_close(self):
         self.end()
@@ -58,13 +62,18 @@ class Circle(sge.Object):
 
         # Set view
         my_view = sge.game.current_room.views[0]
+        zoom_add = (sge.keyboard.get_pressed("hyphen") -
+                    sge.keyboard.get_pressed("equals"))
+        my_view.width += zoom_add
+        my_view.width = max(48, min(my_view.width, sge.game.current_room.width))
+        my_view.height = my_view.width
         my_view.x = self.x - (my_view.width // 2)
         my_view.y = self.y - (my_view.height // 2)
 
 
 def main():
     # Create Game object
-    Game(width=320, height=240, scale_smooth=True,
+    Game(width=240, height=240, scale_smooth=True,
          collision_events_enabled=False)
 
     # Load sprites
@@ -80,10 +89,10 @@ def main():
     objects =  [circle]
 
     # Create views
-    views = [sge.View(0, 0, 0, 0, 320, 240)]
+    views = [sge.View(0, 0, 0, 0, 240, 240, 240, 240)]
 
     # Create rooms
-    sge.Room(objects, 640, 480, views=views, background=background)
+    sge.Room(objects, 640, 640, views=views, background=background)
 
     sge.game.start()
 
