@@ -26,6 +26,7 @@ import warnings
 import pygame
 
 import sge
+from sge import six
 
 
 __all__ = ['Game']
@@ -361,7 +362,7 @@ class Game(object):
         # Setup sound channels
         self._available_channels = []
         if pygame.mixer.get_init():
-            for i in range(pygame.mixer.get_num_channels()):
+            for i in six.moves.range(pygame.mixer.get_num_channels()):
                 self._available_channels.append(pygame.mixer.Channel(i))
 
         # Setup joysticks
@@ -421,6 +422,8 @@ class Game(object):
             self.music = self._start_music
             self.objects = self._start_objects
             self.rooms = self._start_rooms
+            self._colliders = []
+            self._alarms = {}
 
             for i in self.objects:
                 obj = self.objects[i]
@@ -439,10 +442,18 @@ class Game(object):
                 obj.bbox_height = self._object_start_bbox_height[obj.id]
                 obj.collision_ellipse = self._object_start_collision_ellipse[obj.id]
                 obj.collision_precise = self._object_start_collision_precise[obj.id]
+                obj._alarms = {}
+
+                if obj.tangible:
+                    self._colliders.append(obj)
 
             for room in self.rooms:
                 room._reset()
 
+            # Stop all sounds
+            sge.Music.stop()
+            sge.Sound.stop_all()
+
             self.rooms[0].start()
         else:
             self._running = True
@@ -668,16 +679,15 @@ class Game(object):
                 if self.collision_events_enabled:
                     # Set objects' colliders
                     room = self.current_room
-                    for ref in self._colliders:
-                        obj = ref()
-                        if obj is not None:
+                    for obj in self._colliders:
+                        if obj in room.objects:
                             obj._colliders = []
                             for area in obj._collision_areas:
                                 if area is not None:
                                     i, j = area
-                                    room_area = room._collision_areas[i][j]
+                                    room_area = sge.collision_areas[i][j]
                                 else:
-                                    room_area = room._collision_area_void
+                                    room_area = sge.collision_area_void
 
                                 for other in room_area:
                                     if (other is not obj and
@@ -685,8 +695,7 @@ class Game(object):
                                         obj._colliders.append(other)
 
                     # Detect collisions
-                    for ref in self._colliders:
-                        obj = ref()
+                    for obj in self._colliders:
                         if obj is not None:
                             obj._detect_collisions()
 
@@ -929,7 +938,8 @@ class Game(object):
             elif event.type == pygame.JOYHATMOTION:
                 jsname = self._js_names[event.joy]
                 input_event = sge.input.JoystickHatMove(
-                    jsname, event.joy, event.hat, *event.value)
+                    jsname, event.joy, event.hat, event.value[0],
+                    -event.value[1])
                 self.input_events.append(input_event)
             elif event.type == pygame.JOYBALLMOTION:
                 jsname = self._js_names[event.joy]
@@ -1056,16 +1066,24 @@ class Game(object):
                 ystart = y
 
                 if layer.xrepeat and layer.yrepeat:
-                    for y in range(ystart, view.y + view.height, img_h):
-                        for x in range(xstart, view.x + view.width, img_w):
+                    for y in six.moves.range(int(round(ystart)),
+                                             int(round(view.y + view.height)),
+                                             img_h):
+                        for x in six.moves.range(
+                                int(round(xstart)),
+                                int(round(view.x + view.width)), img_w):
                             images.append((img, x, y, layer.z, None))
                 elif (layer.xrepeat and y < view.y + view.height and
                       y + img_h > view.y):
-                    for x in range(xstart, view.x + view.width, img_w):
+                    for x in six.moves.range(int(round(xstart)),
+                                             int(round(view.x + view.width)),
+                                             img_w):
                         images.append((img, x, y, layer.z, None))
                 elif (layer.yrepeat and x < view.x + view.width and
                       x + img_w > view.x):
-                    for y in range(ystart, view.y + view.height, img_h):
+                    for y in six.moves.range(int(round(ystart)),
+                                             int(round(view.y + view.height)),
+                                             img_h):
                         images.append((img, x, y, layer.z, None))
                 elif (x < view.x + view.width and x + img_w > view.x and
                       y < view.y + view.height and y + img_h > view.y):
@@ -1790,8 +1808,8 @@ class Game(object):
 
             # Decide window size
             if self.scale:
-                self._window_width = self.width * self.scale
-                self._window_height = self.height * self.scale
+                self._window_width = int(self.width * self.scale)
+                self._window_height = int(self.height * self.scale)
             else:
                 self._xscale = self._window_width / self.width
                 self._yscale = self._window_height / self.height
@@ -1949,7 +1967,7 @@ class Game(object):
         new_num_channels = old_num_channels + 4
         pygame.mixer.set_num_channels(new_num_channels)
 
-        for i in range(old_num_channels, new_num_channels):
+        for i in six.moves.range(old_num_channels, new_num_channels):
             self._available_channels.append(pygame.mixer.Channel(i))
 
     def _handle_music(self):
