@@ -24,11 +24,11 @@ import os
 import sys
 import traceback
 import math
-import weakref
 
 import pygame
 
 import sge
+from sge import six
 
 
 __all__ = ['Object', 'Mouse']
@@ -275,6 +275,12 @@ class Object(object):
 
        The unique identifier for this object.  (Read-only)
 
+    .. attribute:: alive
+
+       Whether or not the object is alive.  An object is considered to
+       be "alive" from the time it is first added to a room to the time
+       it is destroyed.  (Read-only)
+
     .. attribute:: mask
 
        The current mask used for non-rectangular collision detection.
@@ -350,18 +356,19 @@ class Object(object):
     @tangible.setter
     def tangible(self, value):
         self._tangible = value
-        position = None
-        for i in range(len(sge.game._colliders)):
-            if sge.game._colliders[i]() is self:
-                position = i
-                break
 
         if self.tangible:
-            if position is None:
-                sge.game._colliders.append(weakref.ref(self))
+            if self not in sge.game._colliders:
+                gcolliders = sge.game._colliders[:]
+                gcolliders.append(self)
+                sge.game._colliders = gcolliders
         else:
-            if position is not None:
-                del sge.game._colliders[position]
+            gcolliders = sge.game._colliders[:]
+            while self in gcolliders:
+                gcolliders.remove(self)
+            sge.game._colliders = gcolliders
+
+        self._update_collision_areas()
 
     @property
     def bbox_x(self):
@@ -456,8 +463,8 @@ class Object(object):
     @speed.setter
     def speed(self, value):
         self._speed = value
-        self._xvelocity = math.cos(radians(self.move_direction)) * value
-        self._yvelocity = math.sin(radians(self.move_direction)) * value
+        self._xvelocity = math.cos(math.radians(self.move_direction)) * value
+        self._yvelocity = -math.sin(math.radians(self.move_direction)) * value
 
     @property
     def move_direction(self):
@@ -466,8 +473,8 @@ class Object(object):
     @move_direction.setter
     def move_direction(self, value):
         self._move_direction = value
-        self._xvelocity = math.cos(radians(value)) * self.speed
-        self._yvelocity = math.sin(radians(value)) * self.speed
+        self._xvelocity = math.cos(math.radians(value)) * self.speed
+        self._yvelocity = -math.sin(math.radians(value)) * self.speed
 
     @property
     def image_index(self):
@@ -593,19 +600,19 @@ class Object(object):
                     self.image_rotation)
             elif self.collision_ellipse:
                 # Elliptical mask based on bounding box.
-                mask = [[False for y in range(self.bbox_height)]
-                        for x in range(self.bbox_width)]
+                mask = [[False for y in six.moves.range(self.bbox_height)]
+                        for x in six.moves.range(self.bbox_width)]
                 a = len(mask) / 2
                 b = len(mask[0]) / 2 if mask else 0
 
-                for x in range(len(mask)):
-                    for y in range(len(mask[x])):
+                for x in six.moves.range(len(mask)):
+                    for y in six.moves.range(len(mask[x])):
                         if ((x - a) / a) ** 2 + ((y - b) / b) ** 2 <= 1:
                             mask[x][y] = True
             else:
                 # Mask is all pixels in the bounding box.
-                mask = [[True for y in range(self.bbox_height)]
-                        for x in range(self.bbox_width)]
+                mask = [[True for y in six.moves.range(self.bbox_height)]
+                        for x in six.moves.range(self.bbox_width)]
 
             self._masks[id_] = mask
             return mask
@@ -613,42 +620,44 @@ class Object(object):
     @property
     def mask_x(self):
         if self.collision_precise:
+            origin_x = self.image_origin_x
             id_ = (self.sprite.id, self.sprite.width, self.sprite.height,
-                   self.image_origin_x, self.image_origin_y, self.image_xscale,
+                   origin_x, self.image_origin_y, self.image_xscale,
                    self.image_yscale, self.image_rotation)
             if id_ in self._masks_x_offset:
                 offset = self._masks_x_offset[id_]
-                return self.x - (self.image_origin_x + offset)
+                return self.x - (origin_x + offset)
             elif self.image_rotation % 180:
                 width = self._get_image_width()
                 normal_width = self._get_normal_image_width()
                 offset = (width - normal_width) / 2
                 self._masks_x_offset[id_] = offset
-                return self.x - (self.image_origin_x + offset)
+                return self.x - (origin_x + offset)
             else:
                 self._masks_x_offset[id_] = 0
-                return self.x - self.image_origin_x
+                return self.x - origin_x
         else:
             return self.bbox_left
 
     @property
     def mask_y(self):
         if self.collision_precise:
+            origin_y = self.image_origin_y
             id_ = (self.sprite.id, self.sprite.width, self.sprite.height,
-                   self.image_origin_x, self.image_origin_y, self.image_xscale,
+                   self.image_origin_x, origin_y, self.image_xscale,
                    self.image_yscale, self.image_rotation)
             if id_ in self._masks_y_offset:
                 offset = self._masks_y_offset[id_]
-                return self.y - (self.image_origin_y + offset)
+                return self.y - (origin_y + offset)
             elif self.image_rotation % 180:
                 height = self._get_image_height()
                 normal_height = self._get_normal_image_height()
                 offset = (height - normal_height) / 2
                 self._masks_y_offset[id_] = offset
-                return self.y - (self.image_origin_y + offset)
+                return self.y - (origin_y + offset)
             else:
                 self._masks_y_offset[id_] = 0
-                return self.y - self.image_origin_y
+                return self.y - origin_y
         else:
             return self.bbox_top
 
@@ -674,17 +683,52 @@ class Object(object):
         more information.
 
         """
+        self._x = x
+        self._y = y
+        self.z = z
+        self.active = active
+        self.checks_collisions = checks_collisions
+        self.regulate_origin = regulate_origin
+        self.collision_ellipse = collision_ellipse
+        self.collision_precise = collision_precise
+        self._xvelocity = 0
+        self._yvelocity = 0
+        self._move_direction = 0
+        self._speed = 0
+        self._image_index = image_index
+        self.image_origin_x = image_origin_x
+        self.image_origin_y = image_origin_y
+        self.image_xscale = image_xscale
+        self.image_yscale = image_yscale
+        self.image_rotation = image_rotation
+        self.image_alpha = image_alpha
+        self.image_blend = image_blend
+        self.alive = False
         self.xstart = x
         self.ystart = y
         self.xprevious = x
         self.yprevious = y
+        self._anim_count = 0
+        self._origins_x = {}
+        self._origins_y = {}
+        self._collision_areas = []
+        self._colliders = []
+        self._masks = {}
+        self._masks_x_offset = {}
+        self._masks_y_offset = {}
+        self._alarms = {}
+
+        if ID is None:
+            ID = 0
+            while ID in sge.game.objects:
+                ID += 1
+        self.id = ID
+
+        objects = sge.game.objects.copy()
+        objects[self.id] = self
+        sge.game.objects = objects
 
-        self._image_index = image_index
         self.sprite = sprite
-        self.visible = visible
-        self.active = active
-        self.checks_collisions = checks_collisions
-        self.tangible = tangible
         if self.sprite is not None:
             sprite_bbox_x = self.sprite.bbox_x
             sprite_bbox_y = self.sprite.bbox_y
@@ -701,67 +745,16 @@ class Object(object):
                             sprite_bbox_width)
         self._bbox_height = (bbox_height if bbox_height is not None else
                              sprite_bbox_height)
-        self.regulate_origin = regulate_origin
-        self.collision_ellipse = collision_ellipse
-        self.collision_precise = collision_precise
-        self._collision_areas = []
-        self._colliders = []
 
-        objects = sge.game.objects.copy()
-        if ID is not None:
-            self.id = ID
-        else:
-            ID = 0
-            while ID in objects:
-                ID += 1
-            self.id = ID
-
-        objects[self.id] = self
-        sge.game.objects = objects
-
-        self._x = x
-        self._y = y
-        self.z = z
-        self._xvelocity = 0
-        self._yvelocity = 0
-        self._move_direction = 0
-        self._speed = 0
         self.xvelocity = xvelocity
         self.yvelocity = yvelocity
-        self._anim_count = 0
-        self._origins_x = {}
-        self._origins_y = {}
-        self.image_origin_x = image_origin_x
-        self.image_origin_y = image_origin_y
+        self.tangible = tangible
+        self.visible = visible
         self.image_fps = image_fps
-        self.image_xscale = image_xscale
-        self.image_yscale = image_yscale
-        self.image_rotation = image_rotation
-        self.image_alpha = image_alpha
-        self.image_blend = image_blend
-        self._masks = {}
-        self._masks_x_offset = {}
-        self._masks_y_offset = {}
-
-        self._alarms = {}
-
         self._rect = pygame.Rect(self.bbox_x, self.bbox_y, self.bbox_width,
                                  self.bbox_height)
 
-        self.z = z
-
-        self._start_x = self.x
-        self._start_y = self.y
-        self._start_z = self.z
-        self._start_sprite = self.sprite
-        self._start_visible = self.visible
-        self._start_checks_collisions = self.checks_collisions
-        self._start_bbox_x = self.bbox_x
-        self._start_bbox_y = self.bbox_y
-        self._start_bbox_width = self.bbox_width
-        self._start_bbox_height = self.bbox_height
-        self._start_collision_ellipse = self.collision_ellipse
-        self._start_collision_precise = self.collision_precise
+        self._update_collision_areas()
 
     def collision(self, other=None, x=None, y=None):
         """Return a list of objects colliding with this object.
@@ -786,7 +779,7 @@ class Object(object):
           :const:`None`, :attr:`y` will be used.
 
         """
-        if self.tangible and other.tangible:
+        if self.tangible:
             room = sge.game.current_room
             others = []
             collisions = []
@@ -794,13 +787,12 @@ class Object(object):
             for area in self._collision_areas:
                 if area is not None:
                     i, j = area
-                    room_area = room._collision_areas[i][j]
+                    room_area = sge.collision_areas[i][j]
                 else:
-                    room_area = room._collision_area_void
+                    room_area = sge.collision_area_void
 
-                for ref in room_area:
-                    obj = ref()
-                    if obj is not None and obj is not self:
+                for obj in room_area:
+                    if obj is not self and obj.tangible:
                         if other is None:
                             others.append(obj)
                         elif isinstance(other, Object):
@@ -881,39 +873,44 @@ class Object(object):
             return None
 
     def destroy(self):
-        """Destroy the object."""
-        self.event_destroy()
+        """Destroy the object.
+
+        This removes the object from all rooms in the game, deletes its
+        reference in :attr:`sge.game.objects`, and marks it as "dead".
+
+        """
+        self.alive = False
+
+        for room in sge.game.rooms:
+            room.remove(self)
+
+        gcolliders = sge.game._colliders[:]
+        while self in gcolliders:
+            gcolliders.remove(self)
+        sge.game._colliders = gcolliders
+
         objects = sge.game.objects.copy()
         if self.id in objects:
             del objects[self.id]
         sge.game.objects = objects
 
-        for room in sge.game.rooms:
-            objects = room.objects[:]
-            while self in objects:
-                objects.remove(self)
-            room.objects = objects
-
-            objects_by_class = room.objects_by_class.copy()
-            for cls in objects_by_class:
-                if isinstance(self, cls):
-                    objects = objects_by_class[cls][:]
-                    while self in objects:
-                        objects.remove(self)
-                    objects_by_class[cls] = objects
-            room.objects_by_class = objects_by_class
+        self.event_destroy()
 
     def event_create(self):
         """Create event.
 
-        Called when the object is created.  It is always called after
-        any room start events occurring at the same time.
+        Called right after the object is created.  It is always called
+        after any room start events occurring at the same time.
 
         """
         pass
 
     def event_destroy(self):
-        """Destroy event."""
+        """Destroy event.
+
+        Called right after the object is destroyed.
+
+        """
         pass
 
     def event_begin_step(self, time_passed, delta_mult):
@@ -1438,9 +1435,9 @@ class Object(object):
             self.event_update_position(delta_mult)
 
     def _update_collision_areas(self):
-        if self.tangible:
-            room = sge.game.current_room
-
+        if (self.alive and self.tangible and
+                sge.game.current_room is not None and
+                self in sge.game.current_room.objects):
             if self.collision_precise:
                 my_areas = sge.collision._get_rectangle_collision_areas(
                     self.mask_x, self.mask_y, len(self.mask),
@@ -1449,45 +1446,48 @@ class Object(object):
                 my_areas = sge.collision._get_rectangle_collision_areas(
                     self.bbox_left, self.bbox_top, self.bbox_width,
                     self.bbox_height)
+        else:
+            my_areas = []
 
+        if my_areas != self._collision_areas:
             for area in self._collision_areas:
                 if area not in my_areas:
                     if area is not None:
                         i, j = area
-                        room_area = room._collision_areas[i][j]
+                        ca = sge.collision_areas[i][j][:]
+                        while self in ca:
+                            ca.remove(self)
+                        sge.collision_areas[i][j] = ca
                     else:
-                        room_area = room._collision_area_void
-
-                    for i in range(len(room_area), 0, -1):
-                        if room_area[i - 1]() is self:
-                            del room_area[i - 1]
+                        ca = sge.collision_area_void[:]
+                        while self in ca:
+                            ca.remove(self)
+                        sge.collision_area_void = ca
 
             for area in my_areas:
                 if area not in self._collision_areas:
                     if area is not None:
                         i, j = area
-                        room._collision_areas[i][j].append(weakref.ref(self))
+                        ca = sge.collision_areas[i][j][:]
+                        ca.append(self)
+                        sge.collision_areas[i][j] = ca
                     else:
-                        room._collision_area_void.append(weakref.ref(self))
+                        ca = sge.collision_area_void[:]
+                        ca.append(self)
+                        sge.collision_area_void = ca
 
-            self._collision_areas = my_areas
-        else:
-            self._collision_areas = []
+        self._collision_areas = my_areas
 
     def _detect_collisions(self):
         if self.checks_collisions:
-            for other_ref in self._colliders:
-                if other_ref() is not None and other_ref() is not self:
-                    other = other_ref()
-                else:
+            for other in self._colliders:
+                if other is self:
                     continue
 
                 # Delete self from the other object's list of colliders to
                 # prevent redundancy.
-                for i in range(len(other._colliders)):
-                    if other._colliders[i]() is self:
-                        del other._colliders[i]
-                        break
+                while self in other._colliders:
+                    other._colliders.remove(self)
 
                 if self.collision(other):
                     self_prev_bbox_left = self.xprevious + self.bbox_x
@@ -1599,41 +1599,9 @@ class Object(object):
     def _set_speed(self):
         # Set the speed and move direction based on xvelocity and
         # yvelocity.
-        self._speed = math.sqrt(self._xvelocity ** 2 + self._yvelocity ** 2)
-
-        if self._yvelocity == 0:
-            base_angle = 0
-        elif self._xvelocity == 0:
-            base_angle = 90
-        else:
-            base_angle = math.degrees(math.atan(abs(self._yvelocity) /
-                                                abs(self._xvelocity)))
-
-        if self._xvelocity < 0 and self._yvelocity < 0:
-            self._move_direction += 180
-        elif self._xvelocity < 0:
-            self._move_direction = 180 - base_angle
-        elif self._yvelocity < 0:
-            self._move_direction = 360 - base_angle
-        else:
-            self._move_direction = base_angle
-
-        self._move_direction %= 360
-
-    def _reset(self):
-        # Reset the object back to its original state.
-        self.x = self._start_x
-        self.y = self._start_y
-        self.z = self._start_z
-        self.sprite = self._start_sprite
-        self.visible = self._start_visible
-        self.detects_collisions = self._start_detects_collisions
-        self.bbox_x = self._start_bbox_x
-        self.bbox_y = self._start_bbox_y
-        self.bbox_width = self._start_bbox_width
-        self.bbox_height = self._start_bbox_height
-        self.collision_ellipse = self._start_collision_ellipse
-        self.collision_precise = self._start_collision_precise
+        self._speed = math.hypot(self._xvelocity, self._yvelocity)
+        self._move_direction = math.degrees(math.atan2(-self._yvelocity,
+                                                       self._xvelocity))
 
 
 class Mouse(Object):
@@ -1706,18 +1674,14 @@ class Mouse(Object):
     @tangible.setter
     def tangible(self, value):
         self._tangible = value
-        position = None
-        for i in range(len(sge.game._colliders)):
-            if sge.game._colliders[i]() is self:
-                position = i
-                break
-
-        if self.tangible:
-            if position is None:
-                sge.game._colliders.append(weakref.ref(self))
+        if value:
+            if self not in sge.game._colliders:
+                sge.game._colliders.append(self)
         else:
-            if position is not None:
-                del sge.game._colliders[position]
+            while self in sge.game._colliders:
+                sge.game._colliders.remove(self)
+
+        self._update_collision_areas()
 
     def __init__(self):
         self._visible = True
